/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
 http://developer.sun.com/berkeley_license.html
 $Id: SessionPasswordStrategy.java,v 1.1 2005/05/17 05:23:51 gmurray71 Exp $ */

package com.sun.j2ee.blueprints.clientstate;

import javax.servlet.http.HttpSession;
/**
 * This strategy generates a password randomly and stores it in the session. 
 * This strategy works in a clustered as well as a non-clustered environment 
 * the session is migrated whenever a server goes down. 
 *
 * @author Inderjeet Singh
 */
public class SessionPasswordStrategy implements PasswordStrategy {
    
    public static final String DEFAULT_SESSION_KEY_FOR_PASSWORD = "com.sun.j2ee.blueprints.clientside-state.password-key";
    public static final int DEFAULT_PASSWORD_LENGTH = 24;
    
    /** @param session HttpSession used to store the password 
     * Uses default values for password length as well as session key to store password
     * @see DEFAULT_SESSION_KEY_FOR_PASSWORD
     * @see DEFAULT_PASSWORD_LENGTH 

     */
    public SessionPasswordStrategy(HttpSession session) {
        this(session, DEFAULT_PASSWORD_LENGTH, DEFAULT_SESSION_KEY_FOR_PASSWORD);
    }
    
    /**
     * @param session HttpSession used to store the password
     * @param passwordLength The desired length of the randomly-generated password 
     * @param sessionKeyForPassword The key used to store the password.
     */
    public SessionPasswordStrategy(HttpSession session, int passwordLength, String sessionKeyForPassword) {
        this.session = session;
        this.passwordLength = passwordLength;
        this.sessionKeyForPassword = sessionKeyForPassword;
    }
    
    /** implementation of the PasswordStrategy method. */
    public String getPassword() {
        String password = (String) session.getAttribute(sessionKeyForPassword);
        if (password == null) {
            password = (String) ByteArrayGuard.getRandomString(passwordLength);
            session.setAttribute(sessionKeyForPassword, password);            
        }
        return password;
    }
    
    private HttpSession session;
    private String sessionKeyForPassword;
    private int passwordLength;
}
