/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://adventurebuilder.dev.java.net/LICENSE.txt
 $Id: ClientStateDeserializer.java,v 1.1 2005/05/17 05:12:34 gmurray71 Exp $ */

package com.sun.j2ee.blueprints.clientstate.deserialize;

import java.io.*;
import java.util.*;

// Apache Commons- Tag-Lib Imports
import org.apache.commons.codec.base64.Base64;

// Import for encryption/decryption routines
import com.sun.j2ee.blueprints.clientstate.ByteArrayGuard;
import com.sun.j2ee.blueprints.clientstate.SessionPasswordStrategy;

// J2EE imports 
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * This class de-serializes Base64 encoded parameters encoded into a web page  *
*/
public class ClientStateDeserializer  {

    public static void deserialize(HttpServletRequest request,
                            HttpServletResponse response)  {
        // de-serialize the request bean
        String beanName = request.getParameter("beanName");
        String secureString = request.getParameter("secure");
        // default to off
        boolean secure = false;
        if (secureString != null) 
            try {
                secure = new Boolean(secureString).booleanValue();
            } catch (Exception ex) {
        }
        if (beanName != null) {
            String key = beanName.trim();
            System.out.println("ClientStateDeserializer: Reconstituting " + key);
            String values = request.getParameter(key);
            byte[] plainData  = Base64.decode(values.getBytes());
            byte[] data;
            if (secure) {
                ByteArrayGuard bag = new ByteArrayGuard(new SessionPasswordStrategy(request.getSession()));
                data = bag.decrypt(plainData);
            } else {
                data = plainData;
            }
                
            try {
                ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(data));
                Object requestObject = requestObject = ois.readObject();
                ois.close();
                // put the de-serialized object back into the request
                System.out.println("ClientStateDeserializer: adding " + beanName + "=" + requestObject);
                request.setAttribute(beanName, requestObject);
            } catch (java.io.OptionalDataException ode) {
                System.err.print("ClientStateDeserializer caught: " + ode);
            } catch (java.lang.ClassNotFoundException cnfe) {
                System.err.print("ClientStateDeserializer caught: " + cnfe);                  
            } catch (java.io.IOException iox) {
                System.err.print("ClientStateDeserializer caught: " + iox);                  
            }
        }
    }
}

