/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
 http://adventurebuilder.dev.java.net/LICENSE.txt
 $Id: ClientStateTag.java,v 1.1 2005/05/17 05:12:35 gmurray71 Exp $ */

package com.sun.j2ee.blueprints.clientstate.serialize;

import java.io.*;
import java.util.*;

// Apache Commons- Tag-Lib Imports
import org.apache.commons.codec.base64.Base64;

// Import for encryption/decryption routines
import com.sun.j2ee.blueprints.clientstate.ByteArrayGuard;
import com.sun.j2ee.blueprints.clientstate.SessionPasswordStrategy;

// J2EE Imports
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import javax.servlet.http.*;

/**
 * This tag saves state in the page and provides a button or
 * image link within a form with the current page parameters
 * and the page request attributes by encoding them as hidden
 * form variables that are serialized using Base 64 Encoded Strings.
 */
public class ClientStateTag extends BodyTagSupport {
        
    private String beanName;
    private String targetURL;
    private boolean secure = false;
    private Class serializableClass = null;
         
    public void setAction(String targetURL) {
        this.targetURL = targetURL;
    }
    
    public void setBeanName(String beanName) {
        this.beanName = beanName;
    }
    
    public void setSecure(boolean secure) {
        this.secure = secure;
    }

    public int doStartTag() throws JspTagException {
        return EVAL_BODY_BUFFERED;
    }
    
    public int doEndTag() throws JspTagException {
        HttpServletRequest request =
                (HttpServletRequest) pageContext.getRequest();
        StringBuffer buffer = new StringBuffer();
        buffer.append("<form method=\"POST\" action=\"" + targetURL + "\">");
        // insert any parameters that may have been added via sub tags

        /**
         *  Serialize the request attribute scoped serialize object
         *  identified by beanName into the page
        */
        
        Object value = request.getAttribute(beanName);
        
        // we need the serializable to do checks
        if (serializableClass == null) {
            try {
                serializableClass = Class.forName("java.io.Serializable");
            } catch (java.lang.ClassNotFoundException cnf) {
                System.err.println("ClientStateTag caught: " + cnf);
            }
        }
        // check if seralizable
        if ((serializableClass != null) && (value != null)) {
            if (serializableClass.isAssignableFrom(value.getClass())) {
                try {
                    ByteArrayOutputStream bos = new ByteArrayOutputStream();
                    ObjectOutput out = new ObjectOutputStream(bos);
                    out.writeObject(value);
                    out.close();
                    byte[] plaindata = bos.toByteArray();
                    byte[] data = null;
                    if (secure) {
                        ByteArrayGuard bag = new ByteArrayGuard(new SessionPasswordStrategy(request.getSession()));
                        bag.encrypt(plaindata);
                        buffer.append(" <input type=\"hidden\" name=\"secure\" value=\"true\" />");
                    } else {
                        data = plaindata;
                    }
                    buffer.append(" <input type=\"hidden\" name=\"beanName\" value=\"" + beanName  + "\" />");
                    buffer.append(" <input type=\"hidden\" name=\"" + beanName + "\" value=\"" +
                                    new String(Base64.encode(data), "ISO-8859-1")  + "\" />");
                } catch (java.io.IOException iox) {
                    System.err.println("ClientStateTag caught IOException: " + iox);
                }
            } else {
                System.out.println(beanName + " is not Serializeable.");
            }
        } else if (value == null) {
            System.out.println(beanName + " does not exist.");
        }
        // put in the body content
        BodyContent bc = getBodyContent();
        buffer.append(bc.getString());
        buffer.append("</form>");
        // write the output to the output stream
        try {
            JspWriter out = pageContext.getOut();
            out.print(buffer.toString());
        } catch (IOException ioe) {
            System.err.println("ClientStateTag: Problems with writing...");
        }
        // reset everything
        beanName = null;
        targetURL = null;
        return EVAL_PAGE;
    }    
}
