/* Copyright 2004-2005 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: http://developer.sun.com/berkeley_license.html
$Id: ConstantMethodBinding.java,v 1.2 2005/03/23 14:49:22 jenniferb Exp $  */

package com.sun.j2ee.blueprints.catalog.validator;

import javax.faces.component.StateHolder;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;

/**
 * This class provides method binding and state holding functionality 
 * used by other classes.
 */

public class ConstantMethodBinding extends MethodBinding
    implements StateHolder {

    private String outcome = null;

    /**
     * Constructs a <code>ConstantMethodBinding</code> object.
     */
     public ConstantMethodBinding() {

    }

    /**
     * Constructs a <code>ConstantMethodBinding</code> object.
     */
    public ConstantMethodBinding(String yourOutcome) {
        outcome = yourOutcome;
    }

    /**
     * Returns the return value resulting from a call to the method identified by 
     * this method binding expression, passing it the specified parameters, relative
     * to the specified <code>FacesContext</code>.
     */  
    public Object invoke(FacesContext context, Object params[]) {
        return outcome;
    }

    /**
     * Returns the Java class respresenting the return type from the method 
     * identified by this method binding expression.
     */
    public Class getType(FacesContext context) {
        return String.class;
    }

    // ----------------------------------------------------- StateHolder Methods


    /**
     * Gets the state of the instance as a serializable object.
     */
    public Object saveState(FacesContext context) {
        return outcome;
    }

    /**
     * Performs any processing required to restore the state from the entries
     * in the <code>state</code> object.
     */
    public void restoreState(FacesContext context, Object state) {
        outcome = (String) state;
    }

    private boolean transientFlag = false;

    /**
     * Returns true if the object implementing this interface must not
     * participate in saving and restoring state.
     */
    public boolean isTransient() {
        return (this.transientFlag);
    }

    /**
     * Denotes whether or not the object implementing this interface must not
     * participate in state saving or restoring.
     */
    public void setTransient(boolean transientFlag) {
        this.transientFlag = transientFlag;
    }

}

