/* Copyright 2005 Sun Microsystems, Inc. All rights reserved.
   You may not modify, use, reproduce, or distribute this software except
   in compliance with the terms of the License at:
       http://developer.sun.com/berkeley_license.html
 */
package com.sun.javaee.blueprints.autocomplete;

import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.faces.context.FacesContext;

import com.sun.javaee.blueprints.components.ui.autocomplete.AutoCompleteUtilities;
import com.sun.javaee.blueprints.components.ui.autocomplete.CompletionResult;


/**
 * This application bean is a managed JSF bean in application scope.
 * When created it will read in a list of US cities and their zip codes.
 * It makes this list available to others who ask for it - such as the
 * AJAX autocompletion servlet.
 *
 * @author Tor Norbye
 * @author Mark Basler
 */
public class ApplicationBean {
    static final boolean bDebug=false;
    private String[] cities;
    private String[] zips;
    private String[] states =
        new String[] {
            "AK", "AL", "AR", "AZ", "CA", "CO", "CT", "DC", "DE", "FL", "GA", "HI", "IA", "ID", "IL",
            "IN", "KS", "KY", "LA", "MA", "MD", "ME", "MI", "MN", "MO", "MS", "MT", "NC", "ND", "NE",
            "NH", "NJ", "NM", "NV", "NY", "OH", "OK", "OR", "PA", "RI", "SC", "SD", "TN", "TX", "UT",
            "VA", "VT", "WA", "WI", "WV", "WY"
        };
        
    /** Creates a new instance of ApplicationBean */
    public ApplicationBean() {
        initializeCities();
    }

    private void initializeCities() {
        InputStream is = null;
        BufferedReader reader=null;
        ArrayList alZip=new ArrayList(), alCities=new ArrayList();

        try {
            
            // read in record
            reader=new BufferedReader(new InputStreamReader(ApplicationBean.class.getResourceAsStream(
                        "/com/sun/javaee/blueprints/autocomplete/cities.data")));
            
            cities = new String[] { "INITIALIZATION ERROR" };
            zips = new String[] { "INITIALIZATION ERROR" };

            String sxLine=null, sxZip=null;
            
            // read in variable amount of data
            // rewrote method, because older version was erroring out
            while((sxLine=reader.readLine()) != null) {
                // loop through are read in zip, cities and states
                // is in the format "00000 City, State"
                if(bDebug) System.out.println("RAW DATA =" + sxLine);
                if(sxLine.length() > 7) {
                    sxZip=sxLine.substring(0,5);
                    // get zip
                    alZip.add(sxZip);

                    // get city & state
                    alCities.add(sxLine.substring(6) + " " + sxZip);
                }
            }
            cities=(String[])alCities.toArray(new String[alCities.size()]);
            zips=(String[])alZip.toArray(new String[alZip.size()]);
            
   
        } catch (IOException ioe) {
            System.out.println("Failed to initialize city datastructures: " + ioe);
            ioe.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }
    }

    /** Return a sorted array of City, State entries. */
    public String[] getCities() {
        // TODO Create full list
        return cities;
    }

    /** Return zip codes corresponding to the City, State array. */
    public String[] getZips() {
        return zips;
    }

    public void completeCity(FacesContext context, String prefix, CompletionResult result) {
        if(bDebug) System.out.println("Completing City - " + prefix);
        AutoCompleteUtilities.addMatchingItems(cities, prefix, result);
    }

    public void completeState(FacesContext context, String prefix, CompletionResult result) {
        if(bDebug) System.out.println("Completing state - " + prefix);
        AutoCompleteUtilities.addMatchingItems(states, prefix, result);
    }
}
