/*
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 * $Id: CatalogFacade.java,v 1.10 2007/04/06 17:51:28 inder Exp $
 */

package com.sun.javaee.blueprints.autoid;

import com.sun.javaee.blueprints.autoid.model.Item;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Resource;
import javax.naming.InitialContext;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;


@PersistenceContext(name="EntityManager", unitName="CatalogPu")
public class CatalogFacade implements ServletContextListener {
    
    public static final boolean SHOW_TYPED_COLLECTION = true;
    public static final boolean SHOW_WILDCARD_COLLECTION = false;        
    
    @PersistenceContext(unitName="CatalogPu")
    private EntityManager emContainerManagedThreadUnsafe;
    
    @PersistenceUnit(unitName="CatalogPu")
    private EntityManagerFactory emf;
    @Resource UserTransaction utx;
    
    public CatalogFacade(){}
    private boolean applicationManagedEntityManager = false;
    private EntityManager createApplicationManagedEntityManagerThroughDependencyInjection() {
        applicationManagedEntityManager = true;
        return emf.createEntityManager();
    }
    
    private EntityManager createContainerManagedEntityManagerThroughJNDI() {
        try {
            applicationManagedEntityManager = false;
            InitialContext ic = new InitialContext();
            return (EntityManager) ic.lookup("java:comp/env/EntityManager");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    private EntityManager getEntityManager() {
        //return emContainerManagedThreadUnsafe;
        //return createContainerManagedEntityManagerThroughJNDI();
        return createApplicationManagedEntityManagerThroughDependencyInjection();
    }
    
    private void rollback(Exception ex) {
        try {
            utx.rollback();
        } catch (SystemException e){
            e.initCause(ex);
            throw new RuntimeException("Error in rollback: ", e);
        }
        throw new RuntimeException("Error persisting item: ", ex);
    }
    
    public void contextDestroyed(ServletContextEvent sce) {
        if (emf.isOpen()) {
            emf.close();
        }
    }
    
    public void contextInitialized(ServletContextEvent sce) {
        ServletContext context = sce.getServletContext();
        context.setAttribute("CatalogFacade", this);
    }
    
    public void addItem(Item item) throws InvalidItemException{
        EntityManager em = getEntityManager();
        if(item.getName().length() == 0) throw new InvalidItemException("The item" +
                " name cannot be empty. Please specify a name for the item. ");
        try{
            utx.begin();
            em.joinTransaction();
            em.persist(item);
            utx.commit();
        } catch(javax.transaction.HeuristicRollbackException hrex) {
            System.err.println("Error in committing item: "+hrex);
            rollback(hrex);
        } catch(javax.transaction.HeuristicMixedException hmex) {
            System.err.println("Error in committing item: "+hmex);
            rollback(hmex);
        } catch(javax.transaction.RollbackException rex) {
            System.err.println("Error in committing item: "+rex);
            rollback(rex);
        } catch(javax.transaction.NotSupportedException nsex) {
            System.err.println("Error in creating transaction item: "+nsex);
            rollback(nsex);
        } catch(SystemException exe){
            System.err.println("Error in creating item: "+exe);
            rollback(exe);
        } finally {
            if (applicationManagedEntityManager) {
                em.close();
            }
        }
    }
    public Item getItem(int itemID){
        EntityManager em = getEntityManager();
        Item item =  em.find(Item.class,itemID);
        if (applicationManagedEntityManager) {
            em.close();
        }
        return item;
    }
    
    public List<String> getItemNamesUsingPlainCollection() {
        List <String> names = new ArrayList<String>();
        List items = getAllItemsInPlainCollection();
        for (Object o : items ) {
            Item i = (Item) o;
            names.add(i.getName());
        }
        return names;
    }
    
    public List<String> getItemNamesUsingTypedCollection() {
        List <String> names = new ArrayList<String>();
        List<Item> items = getAllItemsInTypedCollection();
        for (Item i : items ) {
            names.add(i.getName());
        }
        return names;
    }
    
    @SuppressWarnings("unchecked")
    public List getAllItemsInPlainCollection(){
        EntityManager em = getEntityManager();
        Query query = em.createQuery("SELECT OBJECT(i) FROM Item i");
        List items = query.getResultList();
        if (applicationManagedEntityManager) {
            em.close();
        }
        return items;
    }
    
    @SuppressWarnings("unchecked")
    public List<?> getAllItemsInWildcardCollection(){
        EntityManager em = getEntityManager();
        Query query = em.createQuery("SELECT OBJECT(i) FROM Item i");
        List<Item> items = query.getResultList();
        if (applicationManagedEntityManager) {
            em.close();
        }
        return items;
    }
    
   @SuppressWarnings("unchecked")
    public List<Item> getAllItemsInTypedCollection(){
        EntityManager em = getEntityManager();
        Query query = em.createQuery("SELECT OBJECT(i) FROM Item i");
        List<Item> items = query.getResultList();
        //List<Object> items2 = items;  //results in compile time error
        if (applicationManagedEntityManager) {
            em.close();
        }
        return items;
    }
}
