/*
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 * $Id: CatalogServlet.java,v 1.4 2007/04/06 17:51:28 inder Exp $
 */

package com.sun.javaee.blueprints.autoid;

import java.io.*;
import java.util.*;
import java.text.*;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.annotation.*;
import javax.persistence.*;

import com.sun.javaee.blueprints.autoid.model.*;


/**
 * This is a simple example of an HTTP Servlet.  It responds to the GET
 * method of the HTTP protocol.
 */
public class CatalogServlet extends HttpServlet {
    private CatalogFacade cf;
    private ServletContext context;
    private Map nameSpace;
    
    public void init(ServletConfig config) throws ServletException {        
        context = config.getServletContext();
        cf = (CatalogFacade)context.getAttribute("CatalogFacade");
        initPathMapping();
    }
    
    public void destroy() {
        cf = null;
    }
    
    
    public void doGet(HttpServletRequest request,
            HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("text/html");
        String responseURL = null;
        String fullURL = request.getRequestURI();
        
        // get the screen name
        String selectedURL = null;
        int lastPathSeparator = fullURL.lastIndexOf("/") + 1;
        if (lastPathSeparator != -1) {
            selectedURL = fullURL.substring(lastPathSeparator, fullURL.length());
        }
        responseURL = getResponseURL(selectedURL);
        try{
            if (selectedURL.equals("additem.do")) {
                String desc = request.getParameter("item_desc");
                String name = request.getParameter("item_name");
                String listPrice = request.getParameter("item_listprice");
                String unitCost = request.getParameter("item_unitcost");
                Item item = new Item();
                item.setName(name);
                item.setDescription(desc);
                item.setListPrice(new Float(listPrice));
                item.setUnitCost(new Float(unitCost));
                cf.addItem(item);
                request.setAttribute("result", item.getItemID());
            }
            if (selectedURL.equals("finditem.do")) {
                String id = request.getParameter("item_id");
                Item item = cf.getItem(new Integer(id));
                request.setAttribute("item", item);
            }
            if (selectedURL.equals("findallitems.do") || selectedURL.equals("findallitemsasxml.do")) {
                if (CatalogFacade.SHOW_TYPED_COLLECTION) {
                    List<Item> items = cf.getAllItemsInTypedCollection();
                    request.setAttribute("items", items);
                } else if (CatalogFacade.SHOW_WILDCARD_COLLECTION) {
                    List<?> items = cf.getAllItemsInWildcardCollection();
                    request.setAttribute("items", items);
                } else {
                    // just use the plain collection
                    List items = cf.getAllItemsInPlainCollection();
                    request.setAttribute("items", items);
                }
            }
        } catch (InvalidItemException ixe) {
            System.err.println("CatalogServlet error: "+ixe);
            request.setAttribute("error_message", ixe);
            responseURL = getResponseURL("error.do");
        } catch(NumberFormatException nfe){
            System.err.println("CatalogServlet Number format exception: "+nfe);
            request.setAttribute("error_message", nfe);
            responseURL = getResponseURL("error.do");
        }
        context.getRequestDispatcher(responseURL).forward(request, response);
    }
    protected String getResponseURL(String url) {
        return (String) nameSpace.get(url);
    }
    
    protected void initPathMapping() {
        nameSpace = new HashMap();
        nameSpace.put("enteritemdetails.do", "/add.jsp");
        nameSpace.put("enteritemid.do", "/search.jsp");
        nameSpace.put("additem.do", "/result.jsp");
        nameSpace.put("finditem.do", "/item.jsp");
        nameSpace.put("findallitems.do", "/items.jsp");
        nameSpace.put("error.do", "/error.jsp");
        nameSpace.put("index.do", "/index.jsp");
        nameSpace.put("findallitemsasxml.do", "/itemsxml.jsp");
    }
}
