/* 
* Copyright 2006 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
* http://developer.sun.com/berkeley_license.html
* $Id: CompARenderer.java,v 1.3 2006/06/26 16:29:05 basler Exp $ 
*/ 
/*
 * CompARenderer.java
 *
 * Created on April 28, 2006, 7:22 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.sun.javaee.blueprints.components.ui.example;

import java.io.IOException;
import java.net.URL;
import java.util.Map;
import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;
import javax.faces.context.ResponseWriter;
import javax.el.ValueExpression;

/**
 *
 * @author basler
 */
public class CompARenderer extends Renderer {
    private static final boolean bDebug=false;
    private static final String RENDERED_SCRIPT_KEY = "JSF_EXAMPLE_COMPA";
    private static final String COMPA_SCRIPT_RESOURCE="/compA.js";
    private static final String COMPA_CSS_RESOURCE="/compA.css";
    
    
    /**
     * <p>Encode the end of the RichTextEditor tag with the instance specific javascript and html/p>
     *
     * @param context   <code>FacesContext</code>for the current request
     * @param component <code>UIComponent</code> to be decoded
     */
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        
        if(bDebug) System.out.println("\n *** START encodeEnd");
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        
        UIOutput outComp = (UIOutput)component;
        ResponseWriter writer = context.getResponseWriter();

        // set javascript main script file just once
        renderScriptOnce(writer, component, context);
        String clientId=component.getClientId(context);

        // write out markup for the popup
        writer.startElement("div", component);  
        writer.writeAttribute("id", clientId, "id");
        
        String styleClass=(String)getAttributeValue(context, component, "styleClass");
        if (styleClass != null) {
            // add custom class if necessary
            writer.writeAttribute("class", "bpui_compA_popup " + styleClass, "styleClass");
        } else {
            // put in default class
            writer.writeAttribute("class", "bpui_compA_popup", "styleClass");
        }
        String style=(String)getAttributeValue(context, component, "style");
        if (style != null) {
            writer.writeAttribute("style", style, "style");
        }
        

        // Some say that the markup should be placed in the renderer for the component and all modifications should be handled only through
        // as CSS.  Some also say that the startElement and writeAttribute methods should be used when possible.  I found it much too tedious.
        // So for simplicity I wrote out the markup using the write method.  I preferr to use templates as was done in CompBRenderer, but since
        // these examples are supposed to show the different approaches, I hard coded the markup.
        // ****
        // start inside of popup display
        writer.write("<div class=\"bpui_compA_popTop\">");
        writer.write("    <div class=\"bpui_compA_cornerTL\"><div class=\"bpui_compA_cornerTR\">");
        writer.write("        <center><font color=\"white\"><b><span id=\"" + clientId + "_title\">title</span></b></font></center>");
        writer.write("    </div></div>");
        writer.write("</div>");
        writer.write("<div class=\"bpui_compA_popMid\">");
        writer.write("    <table border=\"0\" style=\"width: 100%\">");
        writer.write("        <tr>");
        writer.write("            <td>");
        writer.write("                <table border=\"0\" bgcolor=\"#ffffff\" cellpadding=\"5\" cellspacing=\"5\">");
        writer.write("                    <tr>");
        writer.write("                        <td><span id=\"" + clientId + "_message\">Value</span></td>");
        writer.write("                    </tr>");
        writer.write("                </table>");
        writer.write("            </td>");
        writer.write("        </tr>");
        writer.write("    </table>");
        writer.write("</div>");
        writer.write("<div class=\"bpui_compA_popBot\">");
        writer.write("    <div class=\"bpui_compA_cornerBL\"><div class=\"bpui_compA_cornerBR\">");
        writer.write("    </div></div>");
        writer.write("</div>");
        // end inside of popup display
        // ****
        
        // get properties from UIOutput that were set by the specific taghandler
        writer.endElement("div");
        
        // get url of AJAX service, add context root so page position doesn't matter
        String url=(String)getAttributeValue(context, component, "url");
        if (url != null) {
            url=context.getExternalContext().getRequestContextPath() + "/" + url;
        }

        writer.write("\n");
        writer.startElement("script", component);  
        writer.writeAttribute("type", "text/javascript", null);
        writer.write("\n");
        writer.write("bpui.compA['" + clientId + "']=new bpui.compA.createPopup('"+ clientId +"', '"+ url +"');\n");
        writer.endElement("script");
        writer.write("\n");

        if(bDebug) System.out.println("\n *** END encodeEnd");
    }
    
    
    /** Render the &lt;script&gt; tag which contains supporting JavaScript
     * for this text field.  This is a common method among the AJAX components from the other samples.
     */
    private void renderScriptOnce(ResponseWriter writer, UIComponent component, FacesContext context)
    throws IOException {
        
        // Store attribute in request map when we've rendered the script such
        // that we only do this once per page
        Map requestMap=context.getExternalContext().getRequestMap();
        Boolean scriptRendered=(Boolean)requestMap.get(RENDERED_SCRIPT_KEY);
        
        if (scriptRendered != null && scriptRendered.equals(Boolean.TRUE)) {
            return;
        }
        
        requestMap.put(RENDERED_SCRIPT_KEY, Boolean.TRUE);
        String contextRoot=context.getExternalContext().getRequestContextPath();
        
        writer.startElement("script", component);
        writer.writeAttribute("type", "text/javascript", null);
        writer.writeAttribute("src", contextRoot + COMPA_SCRIPT_RESOURCE, null);
        writer.endElement("script");
        writer.write("\n");

        writer.startElement("link", component);
        writer.writeAttribute("type", "text/css", null);
        writer.writeAttribute("rel", "stylesheet", null);
        writer.writeAttribute("href", contextRoot + COMPA_CSS_RESOURCE, null);
        writer.endElement("link");
        writer.write("\n");
    }
    
    public static Object getAttributeValue(FacesContext context, UIComponent component, String key) {
        Object value=null;
        try {
            // check value expression
            ValueExpression valuexx=component.getValueExpression(key);     
            if(valuexx != null) {
                value=valuexx.getValue(context.getELContext());        
            } else {
                value=component.getAttributes().get(key);
            }
        } catch(Exception ee) {
            ee.printStackTrace();
        }        
        return value;
    } 
    
}
