/* 
* Copyright 2006 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
* http://developer.sun.com/berkeley_license.html
* $Id: CompBPhaseListener.java,v 1.4 2006/06/26 16:29:05 basler Exp $ 
*/ 
/*
 * CompBPhaseListener.java
 *
 * Created on June 16, 2006, 1:41 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package com.sun.javaee.blueprints.components.ui.example;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;
import javax.el.MethodExpression;

/**
 * <p>
 * Phase listener that responds to requests for the JavaScript files, 
 * images and text templates
 *
 * @author Mark Basler
 */
public class CompBPhaseListener implements PhaseListener {
    
    public static final String PATH_PREFIX = "/META-INF";
    public static final String SCRIPT_SUFFIX = ".js";
    public static final String CSS_SUFFIX = ".css";
    public static final String GIF_SUFFIX = ".gif";
    public static final String JPG_SUFFIX = ".jpg";
    public static final String PNG_SUFFIX = ".png";
    public static final String APP_KEY = "/jsf-example/";
    public static final String ACTION_KEY = "/bpui_compb_action/";
    public static final boolean bDebug=false;
    
    public CompBPhaseListener() {
    }

    /*
     * After phase listener that serves resources
     */
    public void afterPhase(PhaseEvent event) {
        FacesContext context=event.getFacesContext();
        String rootId=context.getViewRoot().getViewId();

        if(bDebug) System.out.println("PhaseListener - Root ID " + rootId);
        int iPos=rootId.indexOf(APP_KEY);
        int iPosx=rootId.indexOf(ACTION_KEY);
        
        // see what suffix is used for mapping to content type
        if (rootId.endsWith(SCRIPT_SUFFIX) && iPos > -1) {
            handleResourceRequest(event, PATH_PREFIX + rootId, "text/javascript");
        } else if (rootId.endsWith(CSS_SUFFIX) && iPos > -1) {
            handleResourceRequest(event, PATH_PREFIX + rootId, "text/css");
        } else if (rootId.endsWith(GIF_SUFFIX) && iPos > -1) {
            handleResourceRequest(event, PATH_PREFIX + rootId, "image/gif");
        } else if (rootId.endsWith(JPG_SUFFIX) && iPos > -1) {
            handleResourceRequest(event, PATH_PREFIX + rootId, "image/jpeg");
        } else if (rootId.endsWith(PNG_SUFFIX) && iPos > -1) {
            handleResourceRequest(event, PATH_PREFIX + rootId, "image/x-png");
        } else if (iPosx > -1) {
            // action to invoke through a deferred method expression
            String methodx="#{" + rootId.substring(iPos + ACTION_KEY.length() + 1) + "}";
            methodx=methodx.replace('/','.');
            if(bDebug) System.out.println("Method expression to call = " + methodx);
            
            try {
                Class[] argTypes = { PhaseEvent.class};
                MethodExpression mex=context.getApplication().getExpressionFactory().createMethodExpression(context.getELContext(),
                        methodx, null, argTypes);
                Object[] args = { event };
                mex.invoke(context.getELContext(), args);
            } catch (Exception e) {
                // Just log exception
                e.printStackTrace();
            }
            
            
        }
    }

    
    /**
     * The URL looks like a request for a resource, such as a JavaScript or CSS file. Write
     * the given resource to the response writer.
     */
    private void handleResourceRequest(PhaseEvent event, String resource, String contentType) {
        
        // get full qualified path of class
        URL sxURL = CompBPhaseListener.class.getResource(resource);
        if(bDebug) System.out.println("Resource url = " + sxURL);
        // check for resource
        if(sxURL == null) return;

        HttpServletResponse response=(HttpServletResponse)event.getFacesContext().getExternalContext().getResponse();
        OutputStream outStream=null;
        try {
            response.setContentType(contentType);
            response.setStatus(200);
            outStream=response.getOutputStream();
            // use util to write binary to output stream
            readWriteBinaryUtil(sxURL, outStream);
        } catch (Exception e) {
            String message = "Can't find resource \"" + resource + "\" in the same directory as the class - " + getClass().getResource("PopupPhaseListener.class");
            System.out.println(message);
            e.printStackTrace();
        } finally {
            try {
                outStream.flush();
                //outStream.close();
            } catch (Exception ee) {}
            event.getFacesContext().responseComplete();
        }
    }

    
    
    public void beforePhase(PhaseEvent event) {
    }

    public PhaseId getPhaseId() {
        return PhaseId.RESTORE_VIEW;
    }

    /**
     * The URL looks like a request for a resource, such as a JavaScript or CSS file. Write
     * the given resource to the response output in binary format (needed for images).
     */
    public static void readWriteBinaryUtil(URL sxURL, OutputStream outStream) throws IOException {
        
        DataOutputStream outData=null;
        DataInputStream inData=null;
        int byteCnt=0;
        byte[] buffer=new byte[4096];
        
        // get full qualified path of class
        if(bDebug) System.out.println("RW Loading - " + sxURL);
        try {
            outData=new DataOutputStream(outStream);
            inData=new DataInputStream(openStream(sxURL));
            
            while ((byteCnt=inData.read(buffer)) != -1) {
                if (outData != null && byteCnt > 0) {
                    outData.write(buffer, 0, byteCnt);
                }
            }
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if(inData != null) {
                    inData.close();
                }
            } catch (IOException ioe) {}
        }
    }    
    
    
    private static InputStream openStream(URL sxURL) throws IOException {
        // set cache to false, so the ./lib/bp-popup-balloon.jar file doesn't get locked on redeploy or undeploy on windows.
        InputStream isx=null;
        if(sxURL!= null) {
            URLConnection urlConn = sxURL.openConnection();
            urlConn.setUseCaches(false);
            isx=urlConn.getInputStream();
        }
        return isx;
    }
    
}
