/* 
* Copyright 2006 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
* http://developer.sun.com/berkeley_license.html
* $Id: CompBRenderer.java,v 1.3 2006/06/26 16:29:05 basler Exp $ 
*/ 
/*
 * CompBRenderer.java
 *
 * Created on April 28, 2006, 7:22 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.sun.javaee.blueprints.components.ui.example;

import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.net.URL;
import java.io.Writer;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.net.URLConnection;

import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;
import javax.faces.context.ResponseWriter;
import javax.faces.context.ResponseStream;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlInputText;
import javax.el.ValueExpression;

/**
 *
 * @author basler
 */
public class CompBRenderer extends Renderer {
    private static final boolean bDebug=false;
    private static final String RENDERED_SCRIPT_KEY = "JSF_EXAMPLE_COMPB";
    private static final String COMPB_SCRIPT_RESOURCE="/jsf-example/compB/compB.js";
    private static final String COMPB_CSS_RESOURCE="/jsf-example/compB/compB.css";
    private static final String COMPB_TEMPLATE_RESOURCE="/META-INF/jsf-example/compB/compB_template.txt";
    
    
    /**
     * <p>Encode the end of the RichTextEditor tag with the instance specific javascript and html/p>
     *
     * @param context   <code>FacesContext</code>for the current request
     * @param component <code>UIComponent</code> to be decoded
     */
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        if(bDebug) System.out.println("\n *** START encodeEnd");
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        
        UIOutput outComp = (UIOutput)component;
        ResponseWriter writer = context.getResponseWriter();

        // set javascript main script file just once
        renderScriptOnce(writer, component, context);
        String clientId=component.getClientId(context);
        
        // write out the popup markup that is stored in a template
        URL urlxx=CompBRenderer.class.getResource(COMPB_TEMPLATE_RESOURCE);
        String sxTemplate=readInFragmentAsString(urlxx);
        
        writer.startElement("div", component);  
        writer.writeAttribute("id", clientId, "id");
        
        String styleClass=(String)getAttributeValue(context, component, "styleClass");
        if (styleClass != null) {
            // add custom class if necessary
            writer.writeAttribute("class", "bpui_compB_popup " + styleClass, "styleClass");
        } else {
            // put in default class
            writer.writeAttribute("class", "bpui_compB_popup", "styleClass");
        }
        String style=(String)getAttributeValue(context, component, "style");
        if (style != null) {
            writer.writeAttribute("style", style, "style");
        }
        
        // populate hashmap with tokens and replacement values
        HashMap hmSub=new HashMap();
        hmSub.put("%%%ID%%%", clientId);
        
        // parse page and put in substitutions, then write to output
        writer.write(parseString(sxTemplate, hmSub, false));        
        
        // get properties from UIOutput that were set by the specific taghandler
        writer.endElement("div");
        
        // get url of AJAX service, add context root so page position doesn't matter
        String url=(String)getAttributeValue(context, component, "url");
        if (url != null) {
            url=context.getExternalContext().getRequestContextPath() + "/" + url;
        }
        
        writer.write("\n");
        writer.startElement("script", component);  
        writer.writeAttribute("type", "text/javascript", null);
        writer.write("\n");
        writer.write("bpui.compB['" + clientId + "']=new bpui.compB.createPopup('"+ clientId +"', '"+ url +"');\n");
        writer.endElement("script");
        writer.write("\n");

        if(bDebug) System.out.println("\n *** END encodeEnd");
    }
    
    
    /** Render the &lt;script&gt; tag which contains supporting JavaScript
     * for this text field.  This is a common method among the AJAX components from the other samples.
     */
    private void renderScriptOnce(ResponseWriter writer, UIComponent component, FacesContext context)
    throws IOException {
        
        // Store attribute in request map when we've rendered the script such
        // that we only do this once per page
        Map requestMap=context.getExternalContext().getRequestMap();
        Boolean scriptRendered=(Boolean)requestMap.get(RENDERED_SCRIPT_KEY);
        
        if (scriptRendered != null && scriptRendered.equals(Boolean.TRUE)) {
            return;
        }
        
        requestMap.put(RENDERED_SCRIPT_KEY, Boolean.TRUE);
        String contextRoot=context.getExternalContext().getRequestContextPath();
        
        writer.startElement("script", component);
        writer.writeAttribute("type", "text/javascript", null);
        writer.writeAttribute("src", contextRoot + "/faces" + COMPB_SCRIPT_RESOURCE, null);
        writer.endElement("script");
        writer.write("\n");

        writer.startElement("link", component);
        writer.writeAttribute("type", "text/css", null);
        writer.writeAttribute("rel", "stylesheet", null);
        writer.writeAttribute("href", contextRoot + "/faces" + COMPB_CSS_RESOURCE, null);
        writer.endElement("link");
        writer.write("\n");
    }
    
    
    
    
    public static Object getAttributeValue(FacesContext context, UIComponent component, String key) {
        Object value=null;
        try {
            // check value expression
            ValueExpression valuexx=component.getValueExpression(key);     
            if(valuexx != null) {
                value=valuexx.getValue(context.getELContext());        
            } else {
                value=component.getAttributes().get(key);
            }
        } catch(Exception ee) {
            ee.printStackTrace();
        }        
        return value;
    } 
    
    private static InputStream openStream(URL sxURL) throws IOException {
        // set cache to false, so the ./lib/bp-popup-balloon.jar file doesn't get locked on redeploy or undeploy on windows.
        InputStream isx=null;
        if(sxURL!= null) {
            URLConnection urlConn = sxURL.openConnection();
            urlConn.setUseCaches(false);
            isx=urlConn.getInputStream();
        }
        return isx;
    }    
    
   /**
    * Performs Substitution on passed in strings.  This method could also use a cache to loop up strings previously used
    *
    * @param sxString String to parse
    * @param hmSub A Hashmap of of keys/variables to change
    * @param bLookup Whether to see if the value is to be looked up in the cache.  If true the sxString is used as a key
    */
    public static String parseString(String sxString, HashMap hmSub, boolean bLookup) {

        int iPos=0;
        String sxStringx="";

        if(bLookup) {
            // pull from cache
            //sxStringx=(String)getPages().get(sxString);
        } else {		    
            sxStringx=sxString;
        }

        if (sxStringx != null) {
            // populate other values that are page specific
            Iterator iter=hmSub.keySet().iterator();
            String sxKey, sxValue;
            while(iter.hasNext()) {

                // get key and vaule for substitution
                sxKey=(String)iter.next();
                sxValue=(String)hmSub.get(sxKey);
                sxStringx=sxStringx.replaceAll(sxKey, sxValue);
            }
        }
        return sxStringx;
    }      
    
    
    /*
     * Reads in a file in for processing, using rendering class location
     *
     * @param sxFile The file name to read in
     * @return File in string format
     */
    public static String readInFragmentAsString(URL sxURL) throws IOException {
        BufferedReader bfReader=null;
        StringBuffer sxOut=new StringBuffer();

        try {
            bfReader=new BufferedReader(new InputStreamReader(openStream(sxURL)));
            
            int byteCnt=0;
            char[] buffer=new char[4096];
            while ((byteCnt=bfReader.read(buffer)) != -1) {
                if (byteCnt > 0) {
                   sxOut.append(buffer, 0, byteCnt);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        } finally {
            if(bfReader != null) {
                try {
                    bfReader.close();
                } catch (Exception ee){}
            }
        }
        
        return sxOut.toString();
    }      
}
