<%-- Copyright 2004-2006 Sun Microsystems, Inc.  All rights reserved.  You may not modify, 
use, reproduce, or distribute this software except in compliance with the terms of the 
License at: 
 http://developer.sun.com/berkeley_license.html
 $Id: index.jsp,v 1.5 2007/03/08 21:54:09 basler Exp $ --%>

<%@ taglib uri="http://java.sun.com/jsf/html" prefix="h" %>
<%@ taglib uri="http://java.sun.com/jsf/core" prefix="f" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %> 
<%@taglib prefix="ui" uri="http://java.sun.com/blueprints/ui" %>

<html>
    <head>
      <title>Address Form Using a AJAX JSF Component</title>
      <script type="text/javascript">
       function extractCity(citystatezip) {
         var index = citystatezip.indexOf(',');
         var nextcity = citystatezip.substring(0, index+4);
         return nextcity;
       }

       function chooseCity(city) {
         var index = city.indexOf(',');
         var state = city.substring(index+2, index+4);
         var zip = city.substring(index+5);
         city = city.substring(0, index);
         //alert("data Received =" + city + " - " + state + " - " + zip);
         document.getElementById('autofillform:cityField').value = city;
         document.getElementById('autofillform:stateField').value = state;
         document.getElementById('autofillform:zipField').value = zip;
       }
       
      </script>
    </head>

    <body style="background: white; margin: 0px; padding: 8px">
    <!-- using 0 margin (browsers tend to default to 8) and using padding instead
       is a workaround for a bug on Safari, where the y-computation in the javascript
       doesn't seem to work right when there's a margin on the body. Works on other browsers
       but this workaround doesn't seem to have any harmful effects. -->

 <f:view>

 <h1>Auto-Completion using JavaServer Faces and AJAX</h1>
 <hr/>
 <p>
 This example shows how you can do real time auto-completion using AJAX interactions.
 </p>
 <p>
 In the form below, enter a U.S. address. Two of the fields, city and state,
 offer auto completion from data residing on the server. Both fields are using
 the custom &lt;ajax:completionField/&gt; component.
 </p>

 <p>
 The state field is using the component in a simple, straighforward way; it
 has a method binding to <code>ApplicationBean.completeState</code> which
 just returns a set of matching states that are displayed.
 </p>

 <p>
 The city field on the other hand provides custom JavaScript handlers which
 take a server completion entry (which contains a city, state and zip code,
 returned by <code>ApplicationBean.completeCity</code>)
 and format the entries shown in the completion to only show city and state,
 and another handler which causes a chosen city to populate all three
 textfields for city, state and zip.
 </p>

 <p>
 An obvious extension to this form would be to make the Zip field make asynchronous
 calls too, and have it populate the city and state field when the zip code is
 modified.
 </p>

<h:form id="autofillform">
   <h:panelGrid cellpadding="5" cellspacing="0" columns="3" style="margin-bottom: 20px">

       <h:outputText value="Name:" />
       <h:inputText size="40" id="nameField" value="#{SessionBean.name}" required="true" />
       <h:message for="nameField" showDetail="false" showSummary="true" style="color: red"/>

       <h:outputText value="Street:" />
       <h:inputText size="40" id="streetField" value="#{SessionBean.street}" required="true" />
       <h:message for="streetField" showDetail="false" showSummary="true" style="color: red"/>

       <h:outputText value="City:" />
       <ui:autoComplete size="40" maxlength="100" id="cityField" 
               completionMethod="#{ApplicationBean.completeCity}" 
               value="#{SessionBean.city}" required="true"
               ondisplay="function(item) { return extractCity(item); }"
               onchoose="function(item) { return chooseCity(item); }"
       />

       <h:message for="cityField" showDetail="false" showSummary="true" style="color: red"/>

       <h:outputText value="State:" />
       <ui:autoComplete size="2"  maxlength="100" id="stateField" 
            completionMethod="#{ApplicationBean.completeState}" 
            value="#{SessionBean.state}" required="true" />
       <h:message for="stateField" showDetail="false" showSummary="true" style="color: red" />

       <h:outputText value="Zip:" />
       <h:inputText size="5" id="zipField" value="#{SessionBean.zip}" required="true" />
       <h:message for="zipField" showDetail="false" showSummary="true" style="color: red" />

   </h:panelGrid>

   <h:commandButton action="confirm" value="Submit" />
  </h:form>

 </f:view>
    <br/><br/><br/>
    <c:if test="${initParam['com.sun.blueprints.LIVE_APPLICATION'] != 'true'}">
        <center>
            This coding example is located in the relative directory <a href="${pageContext.request.contextPath}/faces/index.jsp" target="bpcatalog">
            <i>BPCATALOG_INSTALL_DIR</i>/apps/webtier/bp-auto-complete</a>
            <br/>
        </center>    
    </c:if>
    <jsp:include page="footer.jsp" />    
</body>
</html>  
